#version 400 core

in FRAG_VALUES {
	vec2 FragmentUV;
	vec3 Position;
	vec3 Normal;
	//vec3 LightDirection;
	vec3 View;
	//float Distance;
}frag_in;

struct Light {
	vec3 position;
	vec3 ambColor;
	vec3 difColor;
	float constant;
	float linear;
	float quadratic;
};


out vec4 Color;

uniform Light light;
uniform sampler2D diffuse;

vec4 calcLight();

void main() {
	float distance = distance(light.position, frag_in.Position);
	float attenuation = 1 / (light.constant + light.linear * distance + light.quadratic * distance * distance);
	Color = (calcLight() * texture(diffuse, frag_in.FragmentUV)) * attenuation;
	
	
	//Color = texture(diffuse, frag_in.FragmentUV);
	//Color = vec4(light.position,1);
}

vec4 calcLight() {
	//diffuse
	
	vec3 N = normalize(frag_in.Normal);
	vec3 L = normalize(light.position - frag_in.Position);
	vec3 V = normalize(frag_in.View);

	vec3 diffuse = max(dot(L, N), 0) * light.difColor * 0.4f;
	vec3 H = normalize(L + V);
	vec3 specular = pow(max(dot(N, H), 0), 128.5f) * light.difColor * 0.4f;
	return vec4(light.ambColor * 0.7f + diffuse + specular,1);
}