#version 400 core

in FRAG_VALUES {
	vec2 FragmentUV;
	vec3 Position;
	vec3 Normal;
	vec3 View;
	vec4 vertexColor;
	vec4 clipSpace;
	vec3 toCamVec;
	vec3 fromLightVector;
}frag_in;

out vec4 Color;

uniform sampler2D reflection;
uniform sampler2D refraction;
uniform sampler2D dudvMap;
uniform sampler2D normalMap;
uniform sampler2D depthMap;

uniform float offset;
const float waves = 0.8;
const float shineDamper = 20.0;
const float reflectivity = 0.6;
vec3 lightCol = vec3(1.0,1.0,1.0);

void main() {
	vec2 uv = frag_in.FragmentUV;
	vec2 ndc = (frag_in.clipSpace.xy/frag_in.clipSpace.w)/2.0 + 0.5;
	vec2 refractUV = vec2(ndc.x,ndc.y);
	vec2 reflectUV = vec2(ndc.x,-ndc.y);

	float near = 0.1;
	float far = 100.0;
	float depth = texture(depthMap, refractUV).r;
	float floorDist = 2.0 * near * far / (far + near -(2.0 * depth - 1.0) * (far - near));

	depth = gl_FragCoord.z;
	float waterDist = 2.0 * near * far / (far + near -(2.0 * depth - 1.0) * (far - near));
	float waterDepth = floorDist - waterDist;

	vec2 distUV = texture(dudvMap, vec2(uv.x + offset, uv.y)).rg * 0.1;
	distUV = uv + vec2(distUV.x, distUV.y + offset);
	vec2 dist = (texture(dudvMap, distUV).rg * 2.0 - 1.0) * waves * clamp(waterDepth/20.0, 0.0, 1.0);

	reflectUV += dist;
	reflectUV.x = clamp(reflectUV.x, 0.001, 0.999);
	reflectUV.y = clamp(reflectUV.y, -0.999, -0.001);
	refractUV += dist; 
	refractUV = clamp(refractUV, 0.001, 0.999);
	
	vec4 reflectCol = texture(reflection, reflectUV);
	vec4 refractCol = texture(refraction, refractUV);

	vec4 normalCol = texture(normalMap, distUV);
	vec3 normals = normalize(vec3(normalCol.r * 2.0 - 1.0, normalCol.b * 3.0, normalCol.g * 2.0 - 1.0));	

	vec3 view = normalize(frag_in.toCamVec);
	float refractFactor = dot(view, normals);
	refractFactor = pow(refractFactor,1.5);	
	refractFactor = clamp(refractFactor, 0.0, 1.0);

	vec3 reflectedLight = reflect(normalize(frag_in.fromLightVector), normals);
	float specular = max(dot(reflectedLight, view), 0.0);
	specular = pow(specular, shineDamper);
	vec3 specularHighlights = lightCol * specular * reflectivity;

	Color = mix(reflectCol, refractCol, refractFactor);
	Color = mix(Color, vec4(0.5, 0.2, 0.1, 1.0), 0.2) + vec4(specularHighlights,0.0);
	Color.a = clamp(waterDepth/1.0, 0.0, 1.0);
}

